/***************************************************************************
 *   Copyright (C) 2003 by Moris Ravasio                                   *
 *   moris_ravasio@hotmail.com                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/


#include "kink.h"

KInk::KInk(QWidget* parent, const char* name, WFlags fl)
: frmKink(parent,name,fl)
{
	grpError->hide();
	grpLevels->show();
	//make the window as compact as possible here:
	QSize suggestedSize=sizeHint();
	resize(QSize(width(), suggestedSize.height()));
}

KInk::~KInk()
{
}

/*$SPECIALIZATION$*/
void KInk::cmbDevices_activated( const QString & )
{
  last_port = listed_devices[cmbDevices->currentItem()].device_port;
	last_type = listed_devices[cmbDevices->currentItem()].device_type;
	update_view();
}

void KInk::btnAdd_clicked()
{
}

void KInk::btnDetect_clicked()
{
  // Auto detect printers
	auto_detect();
}

void KInk::btnAbout_clicked()
{
  KAboutApplication about(this, "aboutDlg", true);
	about.exec();
}

void KInk::btnUpdate_clicked()
{
  update_view();
}

void KInk::btnClose_clicked()
{
  close();
}

#include "kink.moc"

/*!
    \fn KInk::auto_detect(void)
 */
void KInk::auto_detect(void)
{
  int callback;
	struct ink_level level;

  // Reset combo box contents
  listed_count = 0;
	cmbDevices->clear();

	// Detect printers on parallel ports (if any)
  for(int i=0; i<PARPORT_LAST_PORT; i++)
	{
	  callback = get_ink_level(PARPORT,i,&level);
		switch(callback)
		{
		  case OK:
			  add_device_to_list(PARPORT,i,level.model);
				break;
			case PRINTER_NOT_SUPPORTED:
			  add_device_to_list(PARPORT,i,(unsigned char *)"Unsupported device");
				break;
		}
	}

	// Detect printers on USB ports (if any)
  for(int i=0; i<USB_LAST_PORT; i++)
	{
	  callback = get_ink_level(USB,i,&level);
		switch(callback)
		{
		  case OK:
			  add_device_to_list(USB,i,level.model);
				break;
			case PRINTER_NOT_SUPPORTED:
			  add_device_to_list(USB,i,(unsigned char *)"Unsupported device");
				break;
		}
	}

	// Are there some printers?
	if(!listed_count)
	{
    // No printers detected: show error
		last_type = -1;
		last_port = -1;
		cmbDevices->setEnabled(false);
		btnUpdate->setEnabled(false);
		show_error(tr2i18n("<b>No printers detected on this system</b><br><br>It seems that there are no printer plugged to this computer. Please verify the connections of your devices; maybe a problem of wrong permissions on the device's port.<br><br>For parallel printers try <b>chmod 666 /dev/lpX</b> and <br><b>chmod 666 /dev/parportX</b> (as root), where X is your printer's parallel port number.<br><br>For USB printers, try <b>chmod 666 /dev/usb/lpX</b> (as root), where X is your printer's USB port number.<br><br>Click on <b>Detect</b> to try again."));
	}
	else
	{
    cmbDevices->setEnabled(true);
		btnUpdate->setEnabled(true);

    // Try to keep the focus on the previously selected device
		bool last_found;

		last_found = false;
		for(int i=0; i<listed_count; i++)
		{
		  if(last_port == listed_devices[i].device_port && last_type == listed_devices[i].device_type)
			{
			  last_found = true;
				cmbDevices->setCurrentItem(i);
				break;
			}
		}

		if(!last_found)
		{
		  cmbDevices->setCurrentItem(0);
			last_port = listed_devices[0].device_port;
			last_type = listed_devices[0].device_type;
		}

	  grpError->hide();
	  grpLevels->show();
		update_view();
	}
}


/*!
    \fn KInk::add_device_to_list(void)
		Add a device to the combo box list
 */
void KInk::add_device_to_list(int type, int port, unsigned char *model)
{
  listed_devices[listed_count].device_type = type;
	listed_devices[listed_count].device_port = port;
	++listed_count;

  cmbDevices->insertItem(QString::fromAscii((const char *)model,strlen(model)) + " on " + (type==PARPORT ? "/dev/lp" : "/dev/usb/lp") + QString::number(port));
}


/*!
    \fn KInk::show_error(void)
 */
void KInk::show_error(QString errStr)
{
  lblErrorDescr->setText(errStr);
	grpLevels->hide();
	grpError->show();
}


/*!
    \fn KInk::update_view(void)
 */
void KInk::update_view(void)
{
	int callback;
	struct ink_level level;

	// Disable all progress bars preventively
	barBlack->setEnabled(false);
	barColor->setEnabled(false);
	barCyan->setEnabled(false);
	barMagenta->setEnabled(false);
	barYellow->setEnabled(false);
	barPhoto->setEnabled(false);
	barPhotoCyan->setEnabled(false);
	barPhotoMagenta->setEnabled(false);
	barPhotoYellow->setEnabled(false);
	barBlack->setProgress(0);
	barColor->setProgress(0);
	barCyan->setProgress(0);
	barMagenta->setProgress(0);
	barYellow->setProgress(0);
	barPhoto->setProgress(0);
	barPhotoCyan->setProgress(0);
	barPhotoMagenta->setProgress(0);
	barPhotoYellow->setProgress(0);

	grpError->hide();
	grpLevels->show();

	callback=get_ink_level(last_type,last_port,&level);
	switch(callback)
	{
	case OK:

	if(level.black == 0 &&
	   level.color == 0 &&
		 level.cyan == 0 &&
		 level.magenta == 0 &&
		 level.yellow == 0 &&
		 level.photo == 0 &&
		 level.photocyan == 0 &&
		 level.photomagenta == 0 &&
		 level.photoyellow == 0)
		 level.type = RESPONSE_INVALID;

	  switch (level.type) {
    case ONE_COLOR_FOUND:
		  barColor->setEnabled(true);
		  barColor->setProgress(level.color);
      break;
    case ONE_BLACK_COLOR_FOUND:
		  barBlack->setEnabled(true);
		  barBlack->setProgress(level.black);
      break;
    case TWO_COLORS_FOUND:
		  barBlack->setEnabled(true);
		  barColor->setEnabled(true);
		  barBlack->setProgress(level.black);
		  barColor->setProgress(level.color);
      break;
    case TWO_PHOTO_COLORS_FOUND:
		  barPhoto->setEnabled(true);
		  barColor->setEnabled(true);
		  barPhoto->setProgress(level.photo);
		  barColor->setProgress(level.color);
      break;
    case THREE_COLORS_FOUND:
		  barCyan->setEnabled(true);
		  barMagenta->setEnabled(true);
		  barYellow->setEnabled(true);
		  barCyan->setProgress(level.cyan);
		  barMagenta->setProgress(level.magenta);
		  barYellow->setProgress(level.yellow);
      break;
    case FOUR_COLORS_FOUND:
  		barBlack->setEnabled(true);
		  barCyan->setEnabled(true);
		  barMagenta->setEnabled(true);
		  barYellow->setEnabled(true);
		  barBlack->setProgress(level.black);
		  barCyan->setProgress(level.cyan);
		  barMagenta->setProgress(level.magenta);
		  barYellow->setProgress(level.yellow);
      break;
    case SIX_COLORS_FOUND:
  		barBlack->setEnabled(true);
		  barPhotoCyan->setEnabled(true);
		  barPhotoMagenta->setEnabled(true);
		  barCyan->setEnabled(true);
		  barMagenta->setEnabled(true);
		  barYellow->setEnabled(true);
		  barBlack->setProgress(level.black);
		  barPhotoCyan->setProgress(level.photocyan);
		  barPhotoMagenta->setProgress(level.photomagenta);
		  barCyan->setProgress(level.cyan);
		  barMagenta->setProgress(level.magenta);
		  barYellow->setProgress(level.yellow);
      break;
    case SEVEN_COLORS_FOUND:
  		barBlack->setEnabled(true);
	  	barPhotoCyan->setEnabled(true);
		  barPhotoMagenta->setEnabled(true);
		  barPhotoYellow->setEnabled(true);
		  barCyan->setEnabled(true);
		  barMagenta->setEnabled(true);
		  barYellow->setEnabled(true);
		  barBlack->setProgress(level.black);
		  barPhotoCyan->setProgress(level.photocyan);
		  barPhotoMagenta->setProgress(level.photomagenta);
		  barPhotoYellow->setProgress(level.photoyellow);
		  barCyan->setProgress(level.cyan);
		  barMagenta->setProgress(level.magenta);
		  barYellow->setProgress(level.yellow);
      break;
		case RESPONSE_INVALID:
		  show_error(tr2i18n("<b>Invalid response read from device</b><br><br>Your printer reported that all ink tanks are empty or your printer is currently switched-off.<br><br>Power-on your device and verify the correct installation of cartdriges, then try again clicking <b>Update</b>."));
		}
		break;

	case DEV_PARPORT_INACCESSIBLE:
	case DEV_LP_INACCESSIBLE:
	case COULD_NOT_GET_DEVICE_ID:
	case DEV_USB_LP_INACCESSIBLE:
	case COULD_NOT_WRITE_TO_PRINTER:
	case COULD_NOT_READ_FROM_PRINTER:
	case ERROR:
	case UNKNOWN_PORT_SPECIFIED:
	case NO_DEVICE_CLASS_FOUND:
	case NO_CMD_TAG_FOUND:
	case NO_PRINTER_FOUND:
	case COULD_NOT_PARSE_RESPONSE_FROM_PRINTER:
	  show_error(tr2i18n("<b>Generic error</b><br><br>An error has occured communicating with the selected device. Click <b>Detect</b> and <b>Update</b> to refresh KInk status and try again. If the problem persists, e-mail me a description of the situation and all that can be helpful to me for reconstructing your execution environment."));
		break;

	case NO_INK_LEVEL_FOUND:
	  show_error(tr2i18n("<b>Cannot read ink level from device</b><br><br>Cannot retrieve ink levels from the selected device. Ensure your printer is switched on and click on <b>Update</b> to try again."));
		break;

	case PRINTER_NOT_SUPPORTED:
	  show_error(tr2i18n("<b>Unsupported printer!</b><br><br>Selected printer is not supported by this version of libinklevel. Take a look to <a href='http://home.arcor.de/markusheinz'>http://home.arcor.de/markusheinz</a> for a complete list of devices supported by your version of libinklevel."));
		break;
  }
}
